package de.w665.sharepulse.db.repo;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import com.rethinkdb.RethinkDB;
import com.rethinkdb.gen.ast.Filter;
import com.rethinkdb.net.Connection;
import de.w665.sharepulse.db.RethinkDBConnector;
import de.w665.sharepulse.db.RethinkDBService;
import de.w665.sharepulse.model.FileUpload;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.lang.reflect.Type;
import java.util.Date;
import java.util.Map;

@Repository
@RequiredArgsConstructor
public class FileUploadRepository {

    private final RethinkDBService rethinkDBService;
    private final RethinkDB r;
    private final Connection connection;
    private final Gson gson;

    @Autowired
    public FileUploadRepository(RethinkDBService rethinkDBService, RethinkDBConnector connector) {
        this.rethinkDBService = rethinkDBService;
        this.r = connector.getR();
        this.connection = connector.getConnection();

        this.gson = new GsonBuilder()
                .setDateFormat("dd-MM-yyyy HH:mm:ss") // date field formatting
                .create();
    }

    public void insertFileUpload(FileUpload fileUpload) {
        // Serialize FileUpload object to JSON
        String json = gson.toJson(fileUpload);

        // Convert JSON string back to Map
        Type type = new TypeToken<Map<String, Object>>(){}.getType();
        Map<String, Object> map = gson.fromJson(json, type);

        // Insert the Map into RethinkDB
        r.db("sharepulse").table("file_uploads").insert(map).run(connection);
    }

    public FileUpload retrieveFileUploadByFileId(String fileId) {
        return r.db("sharepulse").table("file_uploads")
                .filter(r.hashMap("fileId", fileId))
                .run(connection, FileUpload.class)
                .next();
    }
}
